var async = require('async')
  , fs = require('fs')
  , path = require('path')

var Backbone = require('backbone')
  , Handlebars = require('handlebars')
//  , Backbone = require('../lilo')
  , _ = require('underscore')

Backbone.$ = function() {
  return {
    attr: function() {}
  }
}

module.exports = Backbone.View.extend({
  constructor: function(options) {
    this.cid = _.uniqueId('view');
    this._configure(options || {});
    this._ensureElement();
    this.preInitialize.apply(this, arguments);
    this.initialize.apply(this, arguments);
    this.delegateEvents();
  },
  context: function() {
    var output = this.data
    if (this.model) {
      output.model = this.model.toJSON()
    }
    if (this.collection) {
      output.collection = this.collection.toJSON()
    }

    return output
  },
  listenTo: function() {
  },
  preInitialize: function() {
    var that = this
    that.data = {}
    that.events = null
    that.config = {
      layoutPath:  'app/template/layout',
      templatePath:  'app/template',
      layout: 'app'
    }

    if (Backbone.View.config) {
      if (Backbone.View.config.layoutPath) {
        that.config.layoutPath = Backbone.View.config.layoutPath 
      }

      if (Backbone.View.config.templatePath) {
        that.config.templatePath = Backbone.View.config.templatePath 
      }

      if (Backbone.View.config.layout) {
        that.config.layout = Backbone.View.config.layout 
      }
    }
  },
  render: function(callback) {
    var that = this;
    async.auto({
      template: function(subCallback) {
        fs.readFile(
          path.join(
            that.config.templatePath, 
            that.template + '.handlebars'
          ),
          'ascii',
          subCallback
        )
      },
      layout: ['template', function(subCallback, results) {
        if (that.config.layout != null) {
          fs.readFile(
            path.join(
              that.config.layoutPath,
              that.config.layout + '.handlebars'
            ),
            'ascii',
            subCallback
          )
        } else {
          subCallback(null, null)
        }
      }],
    },
    function(err, results) {
      if (err) {
        throw new Error(err.message)
        return callback(err)
      }
      var template = Handlebars.compile(results.template)
      if (results.layout === null) {
        return callback(null, template(that.context()))
      } else {
        layout = Handlebars.compile(results.layout)
        return callback(null, layout({ body: template(that.context()) }))
      }
    })
  },
})
